from typing import Dict

from sohstationviewer.conf.constants import ColorMode
from sohstationviewer.database.process_db import execute_db_dict, execute_db
from sohstationviewer.conf.dbSettings import dbConf


def get_chan_plot_info(org_chan_id: str, data_type: str,
                       color_mode: ColorMode = 'B') -> Dict:
    """
    Given chanID read from raw data file and detected dataType
    Return plotting info from DB for that channel

    :param org_chan_id: channel name read from data source
    :param chan_info: info of the channel read from data source
    :param data_type: type of data
    :param color_mode: B/W
    :return info of channel read from DB which is used for plotting
    """
    chan = org_chan_id
    if org_chan_id.startswith('EX'):
        chan = 'EX?'
    if org_chan_id.startswith('VM'):
        chan = 'VM?'
    if org_chan_id.startswith('MassPos'):
        chan = 'MassPos?'
    if org_chan_id.startswith('DS'):
        chan = 'SEISMIC'
    if org_chan_id.startswith('Event DS'):
        chan = 'Event DS?'
    if org_chan_id.startswith('Disk Usage'):
        chan = 'Disk Usage?'
    if dbConf['seisRE'].match(chan):
        chan = 'SEISMIC'
    # The valueColors for each color mode is stored in a separate column.
    # Seeing as we only need one of these columns for a color mode, we only
    # pull the needed valueColors column from the database.
    value_colors_column = 'valueColors' + color_mode
    o_sql = (f"SELECT channel, plotType, height, unit, linkedChan,"
             f" convertFactor, label, fixPoint, "
             f"{value_colors_column} AS valueColors "
             f"FROM Channels as C, Parameters as P")
    if data_type == 'Unknown':
        sql = f"{o_sql} WHERE channel='{chan}' and C.param=P.param"
    else:
        sql = (f"{o_sql} WHERE channel='{chan}' and C.param=P.param"
               f" and dataType='{data_type}'")
    chan_db_info = execute_db_dict(sql)
    seismic_label = None
    if len(chan_db_info) == 0:
        chan_db_info = execute_db_dict(
            f"{o_sql} WHERE channel='DEFAULT' and C.param=P.param")
    else:
        if chan_db_info[0]['channel'] == 'SEISMIC':
            seismic_label = get_seismic_chan_label(org_chan_id)
        chan_db_info[0]['channel'] = org_chan_id

    chan_db_info[0]['label'] = (
        '' if chan_db_info[0]['label'] is None else chan_db_info[0]['label'])
    chan_db_info[0]['unit'] = (
        '' if chan_db_info[0]['unit'] is None else chan_db_info[0]['unit'])
    chan_db_info[0]['fixPoint'] = (
        0 if chan_db_info[0]['fixPoint'] is None
        else chan_db_info[0]['fixPoint'])
    if chan_db_info[0]['label'].strip() == '':
        chan_db_info[0]['label'] = chan_db_info[0]['channel']
    elif seismic_label is not None:
        chan_db_info[0]['label'] = seismic_label
    else:
        chan_db_info[0]['label'] = '-'.join([chan_db_info[0]['channel'],
                                            chan_db_info[0]['label']])
    if chan_db_info[0]['label'].strip() == 'DEFAULT':
        chan_db_info[0]['label'] = 'DEFAULT-' + org_chan_id
    return chan_db_info[0]


def get_seismic_chan_label(chan_id):
    """
    Get label for chan_id in which data stream can use chan_id for label while
        other seismic need to add coordinate to chan_id for label
    :param chan_id: name of channel
    :return label: label to put in front of the plot of the channel
    """
    if chan_id.startswith("DS"):
        label = chan_id
    else:
        try:
            label = chan_id + '-' + dbConf['seisLabel'][chan_id[-1]]
        except KeyError:
            label = chan_id
    return label


def get_signature_channels():
    """
    return the dict {channel: dataType} in which channel is unique for dataType
    """
    sql = ("SELECT channel, dataType FROM Channels where channel in"
           "(SELECT channel FROM Channels GROUP BY channel"
           " HAVING COUNT(channel)=1)")
    rows = execute_db_dict(sql)
    sign_chan_data_type_dict = {r['channel']: r['dataType'] for r in rows}
    return sign_chan_data_type_dict


def get_color_def():
    sql = "SELECT color FROM TPS_ColorDefinition ORDER BY name ASC"
    rows = execute_db(sql)
    return [r[0] for r in rows]


def get_color_ranges():
    sql = "SELECT name, baseCounts FROM TPS_ColorRange"
    rows = execute_db(sql)
    range_names = [r[0] for r in rows]
    base_counts = [r[1] for r in rows]
    all_square_counts = []
    clr_labels = []
    cnt = 0
    # 7 : number of color definition, not include 'E'
    for idx, bc in enumerate(base_counts):
        all_square_counts.append([0] * 7)
        clr_labels.append(['0 counts'])
        for c_idx in range(1, 7):
            cnt = (bc * 10 ** (c_idx - 1))
            all_square_counts[idx][c_idx] = cnt ** 2
            clr_labels[idx].append("+/- {:,} counts".format(cnt))
        clr_labels[idx].append("> {:,} counts".format(cnt))
    return range_names, all_square_counts, clr_labels
